package flink.zanxiangnet.ad.monitoring.clickhouse.sink;

import flink.zanxiangnet.ad.monitoring.AdStatJob;
import flink.zanxiangnet.ad.monitoring.pojo.entity.CostMinuterDM;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

public class BatchSinkMinute extends RichSinkFunction<CostMinuterDM> {
    private static final Logger log = LoggerFactory.getLogger(BatchSinkMinute.class);

    // 对象锁，防止MaxCompute的 Tunnel对象多次初始化
    private static final Object DUMMY_LOCK = new Object();

    private String sql;
    private Connection connection = null;

    public BatchSinkMinute() {

    }

    @Override
    public void open(Configuration config) throws Exception {
        super.open(config);
        Properties props = new Properties();
        props.load(AdStatJob.class.getResourceAsStream("/application.properties"));
        connection = ClickhouseUtil.getConn(props, "cc-bp11803zbt0oq045io.ads.rds.aliyuncs.com",
                "8123", "data_monitoring");
    }

    @Override
    public void invoke(CostMinuterDM costMinuterDM, Context context) throws SQLException {
        String sql = "insert into data_monitoring.cost_minute " +
                "(dt,\n" +
                "minute,\n" +
                "create_time,\n" +
                "hour,\n" +
                "ad_id,\n" +
                "adgroup_id,\n" +
                "adcreative_id,\n" +
                "account_id,\n" +
                "campaign_id,\n" +
                "cost_minute,\n" +
                "cost_diff,\n" +
                "cost_last_hour,\n" +
                "cost_last_hour_diff,\n" +
                "cost_last_two_hour,\n" +
                "cost_last_two_hour_diff,\n" +
                "cost_last_three_trend,\n" +
                "cost_speed,\n" +
                "view_count_minute,\n" +
                "thousand_display_price_minute,\n" +
                "valid_click_count_minute,\n" +
                "ctr_minute,\n" +
                "cpc_minute,\n" +
                "conversions_count_minute,\n" +
                "conversions_cost_minute,\n" +
                "conversions_rate_minute,\n" +
                "first_day_order_roi_minute,\n" +
                "first_day_order_amount_minute,\n" +
                "first_day_order_count_minute,\n" +
                "web_order_amount_minute,\n" +
                "web_order_cost_minute,\n" +
                "web_order_rate_minute,\n" +
                "web_order_count_minute,\n" +
                "order_roi_minute,\n" +
                "order_unit_price_minute,\n" +
                "from_follow_uv_minute,\n" +
                "from_follow_cost_minute,\n" +
                "from_follow_rate_minute,\n" +
                "web_register_count_minute,\n" +
                "web_register_uv_minute,\n" +
                "web_register_cost_minute) values " +
                "(?,?,?,?,?,?,?,?,?,?,\n" +
                "?,?,?,?,?,?,?,?,?,?,\n" +
                "?,?,?,?,?,?,?,?,?,?,\n" +
                "?,?,?,?,?,?,?,?,?,?,\n)";
        System.out.println(costMinuterDM);
        PreparedStatement preparedStatement = connection.prepareStatement(sql);
        preparedStatement.setString(1, costMinuterDM.dt);
        preparedStatement.setString(2, costMinuterDM.minute);
        preparedStatement.setString(3, costMinuterDM.createTime);
        preparedStatement.setString(4, costMinuterDM.hour);
        preparedStatement.setString(5, costMinuterDM.adId);
        preparedStatement.setString(6, costMinuterDM.adgroupId);
        preparedStatement.setString(7, costMinuterDM.adcreativeId);
        preparedStatement.setString(8, costMinuterDM.accountId);
        preparedStatement.setString(9, costMinuterDM.campaignId);
        preparedStatement.setLong(10, costMinuterDM.costMinute);
        preparedStatement.setLong(11, costMinuterDM.costDiff);
        preparedStatement.setLong(12, costMinuterDM.costLastHour);
        preparedStatement.setLong(13, costMinuterDM.costLastHourDiff);
        preparedStatement.setLong(14, costMinuterDM.costLastTwoHour);
        preparedStatement.setLong(15, costMinuterDM.costLastTwoHourDiff);
        preparedStatement.setLong(16, costMinuterDM.costLastThreeTrend);
        preparedStatement.setDouble(17, costMinuterDM.costSpeed);
        preparedStatement.setLong(18, costMinuterDM.viewCountMinute);
        preparedStatement.setLong(19, costMinuterDM.thousandDisplayPriceMinute);
        preparedStatement.setLong(20, costMinuterDM.validClickCountMinute);
        preparedStatement.setDouble(21, costMinuterDM.ctrMinute);
        preparedStatement.setLong(22, costMinuterDM.cpcMinute);
        preparedStatement.setLong(23, costMinuterDM.conversionsCountMinute);
        preparedStatement.setLong(24, costMinuterDM.conversionsCostMinute);
        preparedStatement.setDouble(25, costMinuterDM.conversionsRateMinute);
        preparedStatement.setDouble(26, costMinuterDM.firstDayOrderRoiMinute);
        preparedStatement.setLong(27, costMinuterDM.firstDayOrderAmountMinute);
        preparedStatement.setLong(28, costMinuterDM.firstDayOrderCountMinute);
        preparedStatement.setLong(29, costMinuterDM.webOrderAmountMinute);
        preparedStatement.setLong(30, costMinuterDM.webOrderCostMinute);
        preparedStatement.setDouble(31, costMinuterDM.webOrderRateMinute);
        preparedStatement.setLong(32, costMinuterDM.webOrderCountMinute);
        preparedStatement.setDouble(33, costMinuterDM.orderRoiMinute);
        preparedStatement.setLong(34, costMinuterDM.orderUnitPriceMinute);
        preparedStatement.setLong(35, costMinuterDM.fromFollowUvMinute);
        preparedStatement.setLong(36, costMinuterDM.fromFollowCostMinute);
        preparedStatement.setDouble(37, costMinuterDM.fromFollowRateMinute);
        preparedStatement.setLong(38, costMinuterDM.webRegisterCountMinute);
        preparedStatement.setLong(39, costMinuterDM.webRegisterUvMinute);
        preparedStatement.setDouble(40, costMinuterDM.webRegisterCostMinute);

        preparedStatement.addBatch();
        long startTime = System.currentTimeMillis();
        int ints[] = preparedStatement.executeBatch();
        connection.commit();
        long endTime = System.currentTimeMillis();
        System.out.println("批量插入耗时: " + (endTime - startTime) + "插入数据数量=" + ints.length);
        //TODO:数据去重有问题，去除掉非最新的数据
        Statement statement_duplicate = connection.createStatement();
        String sql_duplicate = "optimize table data_monitoring.cost_minute final;";
        statement_duplicate.executeQuery(sql_duplicate);
        connection.commit();
        long endTime_dp = System.currentTimeMillis();
        System.out.println("数据清理耗时: " + (endTime_dp - endTime));

    }

    @Override
    public void close() throws Exception {

        super.close();
    }
}
