package flink.zanxiangnet.ad.monitoring.process;

import com.aliyun.odps.Instance;
import com.aliyun.odps.Odps;
import com.aliyun.odps.account.Account;
import com.aliyun.odps.account.AliyunAccount;
import com.aliyun.odps.data.Record;
import com.aliyun.odps.task.SQLTask;
import flink.zanxiangnet.ad.monitoring.AdStatJob;
import flink.zanxiangnet.ad.monitoring.clickhouse.sink.ClickhouseUtil;
import flink.zanxiangnet.ad.monitoring.maxcompute.MaxComputeLog;
import flink.zanxiangnet.ad.monitoring.pojo.entity.*;
import flink.zanxiangnet.ad.monitoring.pojo.properties.ApplicationProperties;
import flink.zanxiangnet.ad.monitoring.util.DateUtil;
import flink.zanxiangnet.ad.monitoring.util.JsonUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.state.ValueState;
import org.apache.flink.api.common.state.ValueStateDescriptor;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

public class CostMinuteProcess extends ProcessWindowFunction<AdStatOfMinuteDWD, CostMinuterDM, Long, TimeWindow> {
    private static final DateTimeFormatter formatForLastReduceKey = DateTimeFormatter.ofPattern("yyyy-MM-dd_HH:mm");
    private Connection connection = null;
    private int minutenow = 1;

    @Override
    public void open(Configuration conf) throws SQLException, ClassNotFoundException, IOException {
        Properties props = new Properties();
        props.load(AdStatJob.class.getResourceAsStream("/application.properties"));
        connection = ClickhouseUtil.getConn(props, "cc-bp11803zbt0oq045io.ads.rds.aliyuncs.com",
                "8123", "data_monitoring");
    }

    //数据格式转换
    public CostMinuterDM datachange(AdStatOfMinuteDWD adStatOfMinuteDWD, CostMinuterDM costMinuterDM) {

        //时间-天
        costMinuterDM.dt = adStatOfMinuteDWD.getStatDay();
        //计划 id
        costMinuterDM.campaignId = adStatOfMinuteDWD.getCampaignId().toString();
        //时间- real
        costMinuterDM.createTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(adStatOfMinuteDWD.getCreateTime());

        //时间-小时
        String tmpHour = new SimpleDateFormat("yyyy-MM-dd HH:00:00").format(new Date(adStatOfMinuteDWD.getStatTime()));
        costMinuterDM.hour = tmpHour;
        //时间-分钟
        String tmpMinute = new SimpleDateFormat("yyyy-MM-dd HH:mm:00").format(new Date(adStatOfMinuteDWD.getStatTime()));
        costMinuterDM.minute = tmpMinute;
        //广告id
        costMinuterDM.adId = adStatOfMinuteDWD.getAdId().toString();
        //广告组id
        costMinuterDM.adgroupId = adStatOfMinuteDWD.getAdgroupId().toString();
        //创意id
        costMinuterDM.adcreativeId = "";
        //账号id
        costMinuterDM.accountId = adStatOfMinuteDWD.getAccountId().toString();
        //当天分钟消耗
        costMinuterDM.costMinute = adStatOfMinuteDWD.getCostMinute();
        //消耗速度-----分钟最小为1，部分分钟的速度会慢
        costMinuterDM.costSpeed = adStatOfMinuteDWD.getCostMinute() / this.minutenow;
        //分钟-总浏览量
        costMinuterDM.viewCountMinute = adStatOfMinuteDWD.getViewCountMinute();
        //分钟-总平均曝光成本
        costMinuterDM.thousandDisplayPriceMinute = adStatOfMinuteDWD.getThousandDisplayPriceMinute();
        //分钟-总点击量
        costMinuterDM.validClickCountMinute = adStatOfMinuteDWD.getValidClickCountMinute();
        //分钟-总平均点击率
        costMinuterDM.ctrMinute = adStatOfMinuteDWD.getCtrMinute();
        //分钟-总点击均价
        costMinuterDM.cpcMinute = adStatOfMinuteDWD.getCpcMinute();
        //分钟-总目标转化量
        costMinuterDM.conversionsCountMinute = adStatOfMinuteDWD.getConversionsCountMinute();
        //分钟-总目标平均转化成本
        costMinuterDM.conversionsCostMinute = adStatOfMinuteDWD.getConversionsCostMinute();
        //分钟-总平均转化率
        costMinuterDM.conversionsRateMinute = adStatOfMinuteDWD.getConversionsRateMinute();
        //TODO:分钟-总首日下单roi
        costMinuterDM.firstDayOrderRoiMinute = 0;
        //分钟-总首日下单金额
        costMinuterDM.firstDayOrderAmountMinute = adStatOfMinuteDWD.getFirstDayOrderAmountMinute();
        //分钟-总首日下单量
        costMinuterDM.firstDayOrderCountMinute = adStatOfMinuteDWD.getFirstDayOrderCountMinute();
        //分钟-总下单金额
        costMinuterDM.webOrderAmountMinute = adStatOfMinuteDWD.getOrderAmountMinute();
        //分钟-总平均下单成本
        costMinuterDM.webOrderCostMinute = adStatOfMinuteDWD.getWebOrderCostMinute();
        //分钟-总平均下单率
        costMinuterDM.webOrderRateMinute = adStatOfMinuteDWD.getOrderRateMinute();
        //TODO:分钟-总平均下单量
        costMinuterDM.webOrderCountMinute = adStatOfMinuteDWD.getOrderCountMinute();
        //分钟-总下单roi
        costMinuterDM.orderRoiMinute = adStatOfMinuteDWD.getOrderRoiMinute();
        //分钟-总平均下单客单价
        costMinuterDM.orderUnitPriceMinute = adStatOfMinuteDWD.getOrderUnitPriceMinute();
        //分钟-总公众号关注量
        costMinuterDM.fromFollowUvMinute = adStatOfMinuteDWD.getOfficialAccountFollowCountMinute();
        //TODO:分钟-总平均公众号关注成本
        costMinuterDM.fromFollowCostMinute = adStatOfMinuteDWD.getOfficialAccountFollowCountMinute() == 0 ? 0 : adStatOfMinuteDWD.getCostMinute() / adStatOfMinuteDWD.getOfficialAccountFollowCountMinute();
        //TODO:分钟-总平均公众号关注率
        costMinuterDM.fromFollowRateMinute = adStatOfMinuteDWD.getOfficialAccountFollowRateMinute();
        //TODO:分钟-总注册数-----下面都有问题
        costMinuterDM.webRegisterCountMinute = adStatOfMinuteDWD.getOfficialAccountRegisterUserCountMinute();
        //分钟-总注册人数
        costMinuterDM.webRegisterUvMinute = adStatOfMinuteDWD.getOfficialAccountRegisterUserCountMinute();
        //分钟-总平均注册成本
        costMinuterDM.webRegisterCostMinute = adStatOfMinuteDWD.getOfficialAccountRegisterCostMinute();
        return costMinuterDM;
    }


    @Override
    public void process(Long elementCount, ProcessWindowFunction<AdStatOfMinuteDWD, CostMinuterDM, Long, TimeWindow>.Context context,
                        Iterable<AdStatOfMinuteDWD> iterable, Collector<CostMinuterDM> collector) throws Exception {
        long beginTime = context.window().getStart();
        LocalDateTime beginDateTime = DateUtil.milliToLocalDateTime(beginTime);
        LocalDate beginDate = beginDateTime.toLocalDate();
        //当前天
        String statDay = DateUtil.formatLocalDate(beginDate);
        System.out.println(statDay);
        //当前小时
        int hourInt = beginDateTime.getHour();
        String hour = beginDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:00:00"));
        String lastHour = beginDateTime.minusHours(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:00:00"));
        String lastTwoHour = beginDateTime.minusHours(2).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:00:00"));
        String lastThreeHour = beginDateTime.minusHours(3).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:00:00"));

        this.minutenow = (beginDateTime.getMinute() % 5) == 0 ? 1 : (beginDateTime.getMinute() % 5);
        long now = System.currentTimeMillis();

        //获取前几分钟
        List<AdStatOfMinuteDWD> adStatOfMinuteDWDlist = new ArrayList<>(24);

        for (AdStatOfMinuteDWD adStatOfMinuteDWD : iterable) {
            adStatOfMinuteDWDlist.add(adStatOfMinuteDWD);
            CostMinuterDM costMinuterDM = new CostMinuterDM();

            if (adStatOfMinuteDWD.getHour() != hourInt) {
                continue;
            }

            String adId = adStatOfMinuteDWD.getAdId().toString();
            String sql = "select " +
                    "if(hour='" + lastHour + "',cost_hour,0) last_hour_cost, " +
                    "if(hour='" + lastTwoHour + "',cost_hour,0) last_two_hour_cost, " +
                    "if(hour='" + lastHour + "',cost_hour,0) - if(hour='" + lastTwoHour + "',cost_hour,0) cost_last_hour_diff, " +
                    "(if(hour='" + lastHour + "',cost_hour,0) - if(hour='" + lastTwoHour + "',cost_hour,0))*(if(hour='" + lastTwoHour + "',cost_hour,0) - if(hour='" + lastThreeHour + "',cost_hour,0)) cost_last_three_trend " +
                    "from data_monitoring.cost_hour ch " +
                    "where dt='" + statDay + "' and ad_id='" + adId + "' ";

            System.out.println(sql);
            Statement statement = connection.createStatement();
            ResultSet rs = statement.executeQuery(sql);
            while (rs.next()) {
                costMinuterDM.costLastHour = rs.getLong(1);
                costMinuterDM.costLastTwoHour = rs.getLong(2);
                costMinuterDM.costLastHourDiff = rs.getLong(3);
                costMinuterDM.costLastThreeTrend = rs.getLong(4);

            }


            CostMinuterDM CostMinuterDM_new = datachange(adStatOfMinuteDWD, costMinuterDM);

            collector.collect(CostMinuterDM_new);
            // System.out.println("costminute_输出：" + JsonUtil.toString(CostMinuterDM_new));
        }
        // System.out.println("costminute_windowCount：" + adStatOfMinuteDWDlist.size());


    }

    @Override
    public void clear(ProcessWindowFunction<AdStatOfMinuteDWD, CostMinuterDM, Long, TimeWindow>.Context context) throws Exception {
    }

}
